package br.usp.cofree.negocio.dao;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.search.FullTextSession;
import org.hibernate.search.Search;

/**
 * This is a generic base class for all DAO's in the project. It's basically a
 * thin wrapper around the JPA-specific API calls.
 * 
 * @param <T>
 *            The persistent class for which the concrete extending DAO is
 *            providing access
 */
public class GenericDAO<T> {

	private SessionFactory sessionFactory;
    protected Class<T> classePersistente;

	private final static Log log = LogFactory.getLog(GenericDAO.class);

	public GenericDAO() {

	}
	
    public GenericDAO(Class<T> classePersistente) {
        this.classePersistente = classePersistente;
    }

	/**
	 * Carrega um determinado objeto a partir do banco de dados.
	 * 
	 * @param <T>
	 *            Tipo Java da entidade.
	 * @param <PK>
	 *            Tipo Java da chave primria.
	 * @param id
	 *            Valor da chave primria.
	 * @return objeto carregado a partir do banco de dados.
	 */

	// @Transactional(readOnly = true)
	public <T, PK extends Serializable> T get(PK id) {
		return (T) classePersistente.cast(getSession().get(classePersistente, id));
	}

	public List<T> limitar(List<T> lista, int primeiro, int count) {
		List<T> resultado = new ArrayList<T>();
		for (int i = primeiro; i < Math.min(primeiro + count, lista.size()); i++) {
			resultado.add(lista.get(i));
		}
		return resultado;
	}

	/**
	 * Carrega todos os objetos de um determinado tipo a partir do banco de
	 * dados.
	 * 
	 * @param <T>
	 *            Tipo Java da entidade.
	 * @return lista de objetos carregados a partir do banco de dados
	 */
	// @Transactional(readOnly = true)
	public <T> List<T> getAll() {
		return getSession().createCriteria(classePersistente).list();
	}

	/**
	 * Salva um objeto no banco de dados.
	 * 
	 * @param <T>
	 *            Tipo Java da entidade.
	 * @param obj
	 *            - objeto a ser persistido
	 */
	// @Transactional(isolation = Isolation.DEFAULT, rollbackFor =
	// Exception.class)
	public <T extends Object> void save(T obj) {
		Session s = getSession();
		s.save(obj);
		s.flush();
	}

	/**
	 * Atualiza um objeto no banco de dados.
	 * 
	 * @param <T>
	 *            Tipo Java da entidade.
	 * @param obj
	 *            - objeto a ser atualizado
	 */
	// @Transactional(isolation = Isolation.DEFAULT, rollbackFor =
	// Exception.class)
	public <T extends Object> void update(T obj) {
		Session s = getSession();
		s.update(obj);
		s.flush();
	}

	/**
	 * Apaga (fisicamente) um determinado objeto do banco de dados.
	 * 
	 * @param <T>
	 *            Tipo Java da entidade.
	 * @param obj
	 *            - objeto a ser apagado
	 */
	// @Transactional(isolation = Isolation.DEFAULT, rollbackFor =
	// Exception.class)
	public <T extends Object> void delete(T obj) {
		Session s = getSession();
		s.delete(obj);
		s.flush();
	}

	/**
	 * Coloca a entidade como apenas de "leitura" na sesso do Hibernate.
	 * 
	 * @param <T>
	 *            Tipo Java da entidade.
	 * @param obj
	 *            a entidade.
	 */
	public <T extends Object> void setReadOnly(T obj) {
		getSession().setReadOnly(obj, true);
	}

	/**
	 * Retorna a sesso atual do SessionFactory atual.
	 * 
	 * @return a sesso.
	 */
	public Session getSession() {
		SessionFactory s = getSessionFactory();
		return s.getCurrentSession();
	}

	/**
	 * Retorna o SessionFactory atual.
	 * 
	 * @return o Sessionfactory.
	 */
	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

}
